@@ -13,6 +13,7 @@ has proc => ( is => 'ro', isa =>, InstanceOf ['Pcore::Util::PM::Proc'], init_arg
 has pid => ( is => 'ro', isa => Int, init_arg => undef );    # real RPC process PID, reported in handshake
 has in  => ( is => 'ro', isa => InstanceOf ['Pcore::AE::Handle'], init_arg => undef );    # process IN channel, we can write
 has out => ( is => 'ro', isa => InstanceOf ['Pcore::AE::Handle'], init_arg => undef );    # process OUT channel, we can read
+has on_finish => ( is => 'rw', isa => Maybe [CodeRef] );
 
 around new => sub ( $orig, $self, @ ) {
     my %args = (
@@ -20,11 +21,12 @@ around new => sub ( $orig, $self, @ ) {
         buildargs => undef,                                                               # class constructor arguments
         scan_deps => 0,
         on_ready  => undef,
+        on_finish => undef,
         splice @_, 2,
     );
 
     # create self instance
-    $self = $self->$orig;
+    $self = $self->$orig( { on_finish => $args{on_finish} } );
 
     # create handles
     my ( $in_r,  $in_w )  = portable_socketpair();
@@ -52,15 +54,15 @@ around new => sub ( $orig, $self, @ ) {
     }
 
     # serialize CBOR + HEX
-    $boot_args = P->data->to_cbor( $boot_args, encode => $DATA_ENC_B64 )->$*;
+    $boot_args = P->data->to_cbor( $boot_args, encode => $DATA_ENC_HEX )->$*;
 
     my $cmd = [];
 
     if ($MSWIN) {
-        push $cmd->@*, $perl, q[-MPcore::Util::PM::RPC::Server -e "" ] . $boot_args;
+        push $cmd->@*, $perl, qq[-MPcore::Util::PM::RPC::Server -e "" $args{class}];
     }
     else {
-        push $cmd->@*, $perl, '-MPcore::Util::PM::RPC::Server', '-e', q[], $boot_args;
+        push $cmd->@*, $perl, '-MPcore::Util::PM::RPC::Server', '-e', q[], $args{class};
     }
 
     # needed for PAR, pass current @INC libs to child process via $ENV{PERL5LIB}
@@ -72,9 +74,12 @@ around new => sub ( $orig, $self, @ ) {
     # create proc
     P->pm->run_proc(
         $cmd,
+        stdin    => 1,
         on_ready => sub ($proc) {
             $self->{proc} = $proc;
 
+            $proc->stdin->push_write( $boot_args . $LF );
+
             # wrap AE handles and perform handshale
             $self->_handshake(
                 {   class     => $args{class},
@@ -89,6 +94,11 @@ around new => sub ( $orig, $self, @ ) {
 
             return;
         },
+        on_finish => sub ($proc) {
+            $self->on_finish->($self) if $self->on_finish;
+
+            return;
+        }
     );
 
     return;
@@ -158,7 +168,7 @@ sub _handshake ( $self, $init, $cb ) {
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ## | Sev. | Lines                | Policy                                                                                                         |
 ## |======+======================+================================================================================================================|
-## |    2 | 122, 132             | ValuesAndExpressions::ProhibitEscapedCharacters - Numeric escapes in interpolated string                       |
+## |    2 | 132, 142             | ValuesAndExpressions::ProhibitEscapedCharacters - Numeric escapes in interpolated string                       |
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ##
 ## -----SOURCE FILTER LOG END-----
