@@ -10,9 +10,16 @@ BEGIN {
     # preload Filter::Crypto::Decrypt to avoid "Can't run with Perl compiler backend" fatal error under crypted PAR
     require Filter::Crypto::Decrypt;
     require CBOR::XS;
-    require MIME::Base64;    ## no critic qw[Modules::ProhibitEvilModules]
 
-    $BOOT_ARGS = CBOR::XS::decode_cbor( MIME::Base64::decode_base64url( shift @ARGV ) );
+    # shift class name
+    shift @ARGV;
+
+    # read and unpack boot args from STDIN
+    $BOOT_ARGS = <>;
+
+    chomp $BOOT_ARGS;
+
+    $BOOT_ARGS = CBOR::XS::decode_cbor( pack 'H*', $BOOT_ARGS );
 
     $0 = $BOOT_ARGS->[0];    ## no critic qw[Variables::RequireLocalizedPunctuationVars]
 
@@ -25,6 +32,7 @@ package                      # hide from CPAN
 use Pcore;
 use Pcore::AE::Handle;
 use if $MSWIN, 'Win32API::File';
+use Pcore::Util::PM::RPC qw[:CONST];
 
 if ($MSWIN) {
     Win32API::File::OsFHandleOpen( *IN,  $BOOT_ARGS->[3], 'r' ) or die $!;
@@ -42,7 +50,9 @@ my $OUT;                                            # write to
 Pcore::AE::Handle->new(
     fh       => \*IN,
     on_error => sub ( $h, $fatal, $msg ) {
-        exit;
+        _on_term();
+
+        return;
     },
     on_connect => sub ( $h, @ ) {
         $IN = $h;
@@ -55,7 +65,9 @@ Pcore::AE::Handle->new(
 Pcore::AE::Handle->new(
     fh       => \*OUT,
     on_error => sub ( $h, $fatal, $msg ) {
-        exit;
+        _on_term();
+
+        return;
     },
     on_connect => sub ( $h, @ ) {
         $OUT = $h;
@@ -64,10 +76,23 @@ Pcore::AE::Handle->new(
     }
 );
 
+# ignore INT
+$SIG->{INT} = AE::signal INT => sub {
+    return;
+};
+
+# term on TERM
+$SIG->{TERM} = AE::signal TERM => sub {
+    _on_term();
+
+    return;
+};
+
 my $RPC;
 my $DEPS    = {};
 my $QUEUE   = {};
 my $CALL_ID = 0;
+my $TERM;
 
 our $CV = AE::cv;
 
@@ -120,6 +145,8 @@ $IN->unshift_read(
 
 $CV->recv;
 
+exit;
+
 sub _get_new_deps {
     my $new_deps;
 
@@ -134,8 +161,31 @@ sub _get_new_deps {
     return $new_deps;
 }
 
+sub _on_term {
+    return if $TERM;
+
+    $TERM = 1;
+
+    $RPC->RPC_ON_TERM if $RPC->can('RPC_ON_TERM');
+
+    return;
+}
+
 sub _on_data ($data) {
-    if ( $data->[0]->{method} ) {
+    if ( $data->[0]->{msg} ) {
+
+        # stop receiving messages in TERM state
+        return if $TERM;
+
+        if ( $data->[0]->{msg} == $RPC_MSG_TERM ) {
+            _on_term();
+        }
+    }
+    elsif ( $data->[0]->{method} ) {
+
+        # stop receiving new calls in TERM state
+        return if $TERM;
+
         _on_call( $data->[0]->{call_id}, $data->[0]->{method}, $data->[1] );
     }
     else {
@@ -204,14 +254,14 @@ sub rpc_call ( $self, $method, $data = undef, $cb = undef ) {
     return;
 }
 
-1;
+1;    ## no critic qw[ControlStructures::ProhibitUnreachableCode]
 ## -----SOURCE FILTER LOG BEGIN-----
 ##
 ## PerlCritic profile "pcore-script" policy violations:
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ## | Sev. | Lines                | Policy                                                                                                         |
 ## |======+======================+================================================================================================================|
-## |    2 | 75, 85               | ValuesAndExpressions::ProhibitEscapedCharacters - Numeric escapes in interpolated string                       |
+## |    2 | 100, 110             | ValuesAndExpressions::ProhibitEscapedCharacters - Numeric escapes in interpolated string                       |
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ##
 ## -----SOURCE FILTER LOG END-----
